import os
from logging import Logger


def dictinct_list(l):
    ds = []
    for item in l:
        if not item in ds:
            ds.append(item)
    return ds

def setAttr(o, name, value):
    o[name] = value
    return o


def delAttr(o, name):
    if name in o:
        del o[name]
        return True
    else:
        return False


def isEmptyDict(dictionary):
    for element in dictionary:
        if element:
            return True
        return False


def delete_drive_leter(path):
    path = path.replace(os.path.sep, os.path.altsep)
    if path.find(':') != -1:
        path = (path.split(':')[1]).replace(os.path.sep, os.path.altsep)
        path = ''.join(path)

    if path.startswith(os.altsep):
        return path[1:]
    return path


def get_drive_leter(path):
    path = path.replace(os.path.sep, os.path.altsep)
    if path.find(':') != -1:
        return f"{(path.split(':')[0])}:"
    return None


def replace_alt_set(path):
    return path.replace(os.altsep, os.sep).replace(f'{os.sep}{os.sep}', os.sep)


def replace_sep(path):
    return path.replace(os.sep, os.altsep)


def del_last_not_digit(str):
    res = ''
    flag = False
    for ch in reversed(str):
        if flag or ch.isdigit():
            res += ch
            flag = True

    return res[::-1]


def str_to_bool(s):
    if s in ['True', 'true']:
        return True
    elif s in ['False', 'false']:
        return False
    else:
        raise ValueError(f'{s} is not a good boolean string')


def bool_to_jsBool(value):
    if value == True:
        return "true"
    return "false"


class StackElementNotExist(Exception):
    def __init__(self, *args, **kwargs):  # real signature unknown
        super().__init__('Соответствие не найдено.')


class MultipleStackElement(Exception):
    def __init__(self, *args, **kwargs):  # real signature unknown
        super().__init__('Не однозначный выбор.')


class Stack:

    def __init__(self, stack=[]):
        self.stack = stack.copy()

    def top(self, index=1):
        if len(self.stack) < index:
            return None
        return self.stack[len(self.stack) - index]

    def pop(self):
        if len(self.stack) < 1:
            return None
        return self.stack.pop()

    def push(self, item, exists_function=None, logger=None):
        if callable(exists_function):
            if exists_function(self.stack, item) == True:
                self.stack.append(item)
                if isinstance(logger, Logger):
                    logger.debug(f'self.stack.append: {item}')
        else:
            self.stack.append(item)
            if isinstance(logger, Logger):
                logger.debug(f'self.stack.append: {item}')

        if isinstance(logger, Logger):
            logger.debug(f'size stack: {len(self.stack)}')

    def size(self):
        return len(self.stack)

    def copy(self):
        return Stack(self.stack)

    def find(self, function):
        return [item for item in self.stack if function(item)]

    def find_one(self, function):
        res = self.find(function=function)
        if len(res) == 0:
            raise StackElementNotExist()
        elif len(res) > 1:
            raise MultipleStackElement()

        return res[0]

    def __str__(self):
        return "\n" + ', '.join([str(item.id) for item in self.stack])






from contextlib import contextmanager

from django.conf import settings
from django.utils import timezone

from isc_common.auth.models.user import User
from isc_common.bit import IsBitOff
from isc_common.models.progresses import Progresses
from isc_common.ws.progressStack import ProgressStack


class ProgressDroped(Exception):
    ...


@contextmanager
def managed_progress(qty, user, id, message='Обработано позиций', title='Обработано позиций', props=0):
    progress = Progress(id=id, qty=qty, user=user, message=message, title=title, props=props)
    try:
        yield progress
        progress.close()
    except Exception as ex:
        progress.close()
        if callable(progress.except_func):
            progress.except_func(progress)
        raise ex


def clean_progresses():
    Progresses.objects.all().delete()


class Progress:
    progress = None
    except_func = None

    def __init__(self, qty, user, id, message='Обработано позиций', title='Обработано позиций', props=0):
        self.message = message
        self.title = title
        self.props = props
        self.id = id

        if isinstance(user, int):
            self.user = User.objects.get(id=user)
        elif not isinstance(user, User):
            raise Exception(f'user must be User instance.')
        else:
            self.user = user
        self.qty = qty

        if self.qty > 0:
            channel = f'common_{self.user.username}'
            self.progress = ProgressStack(
                host=settings.WS_HOST,
                port=settings.WS_PORT,
                channel=channel,
                props=self.props,
                user_id=self.user.id
            )

            self.demand_str = f'<h3>{self.message}</h3>'

            if not self.check_exists():
                self.progress.show(
                    cntAll=qty,
                    id=self.id,
                    label_contents=self.demand_str,
                    title=f'<b>{title}</b>',
                )

                if IsBitOff(props, 1):
                    self.progresses, created = Progresses.objects.get_or_create(
                        id_progress=self.id,
                        user=self.user,
                        defaults=dict(
                            cnt=0,
                            label_contents=self.demand_str,
                            qty=self.qty,
                            message=self.message,
                            props=self.props,
                            title=self.title,
                            lastmodified=timezone.now()
                        )
                    )

        self.cnt = 0

    def check_4_exit(self):
        from isc_common.models.deleted_progresses import Deleted_progresses
        res = 0
        for delted_process in Deleted_progresses.objects.filter(id_progress=self.id, user=self.user):
            res += 1
            delted_process.delete()

        return res

    def check_exists(self):
        res = Progresses.objects.filter(
            id_progress=self.id,
            user=self.user
        ).count() > 0

    def step(self):
        res = 0
        if self.progress != None:
            self.progress.setCntDone(self.cnt, self.id)
            self.cnt += 1
            if self.qty == self.cnt:
                self.cnt = 0
            self.progresses.cnt = self.cnt
            self.progresses.lastmodified = timezone.now()
            self.progresses.save()
            res = self.check_4_exit()
            return res

    def setContentsLabel(self, content):
        if self.progress != None:
            self.progress.setContentsLabel(labelContents=content, id=self.id)
            self.progresses.labelContents = content
            self.progresses.save()

    def sendInfo(self, message):
        if self.progress != None:
            self.progress.sendInfo(message=message)

    def sendWarn(self, message):
        if self.progress != None:
            self.progress.sendWarn(message=message)

    def sendMessage(self, type, message=None):
        if self.progress != None:
            self.progress.sendMessage(type=type, message=message)

    def close(self):
        if self.progress != None:
            self.progress.close(self.id)
            try:
                self.progresses.delete()
            except AssertionError:
                pass



import logging
import sys
import uuid

from django.db import connection

logger = logging.getLogger(__name__)


def create_tmp_mat_view(sql_str, params=[], indexes=[], mat_view_name=None):
    if mat_view_name == None:
        mat_view_name = f'tmp_{str(uuid.uuid4()).replace("-", "_")}'

    if isinstance(indexes, list):
        indexes = [f'CREATE INDEX {mat_view_name}_{index}_idx ON {mat_view_name} USING btree ({index})' for index in indexes]
        suffix = ';'.join(indexes)
    sql_txt = f'''CREATE MATERIALIZED VIEW {mat_view_name} AS {sql_str} WITH DATA; {suffix}'''
    try:
        with connection.cursor() as cursor:
            cursor.execute(sql_txt, params)
        return mat_view_name
    except Exception as ex:
        exc_info = sys.exc_info()
        logger.error(msg=ex, exc_info=exc_info)
        return None


def drop_mat_view(mat_view_name):
    sql_txt = f'DROP MATERIALIZED VIEW IF EXISTS {mat_view_name} CASCADE;'
    try:
        with connection.cursor() as cursor:
            cursor.execute(sql_txt)
        return True
    except Exception as ex:
        exc_info = sys.exc_info()
        logger.error(msg=ex, exc_info=exc_info)
        return False


def refresh_mat_view(mat_view_name):
    sql_txt = f'REFRESH MATERIALIZED VIEW {mat_view_name};'
    try:
        with connection.cursor() as cursor:
            cursor.execute(sql_txt)
        return True
    except Exception as ex:
        exc_info = sys.exc_info()
        logger.error(msg=ex, exc_info=exc_info)
        return False



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from reports.models.jasper_reports import Jasper_reports, Jasper_reportsManager


@JsonResponseWithException()
def Jasper_reports_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Jasper_reports.objects.
                select_related().
                get_range_rows1(
                request=request,
                function=Jasper_reportsManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Jasper_reports_Add(request):
    return JsonResponse(DSResponseAdd(data=Jasper_reports.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Jasper_reports_Update(request):
    return JsonResponse(DSResponseUpdate(data=Jasper_reports.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Jasper_reports_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Jasper_reports.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Jasper_reports_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Jasper_reports.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Jasper_reports_Info(request):
    return JsonResponse(DSResponse(request=request, data=Jasper_reports.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Jasper_reports_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Jasper_reports.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)            
